from plone.caching.lookup import DefaultRulesetLookup
from plone.caching.testing import IMPLICIT_RULESET_REGISTRY_UNIT_TESTING

import unittest
import z3c.caching.registry


class DummyView:
    pass


class DummyResponse(dict):
    def addHeader(self, name, value):
        self.setdefault(name, []).append(value)


class DummyRequest(dict):
    def __init__(self, published, response):
        self["PUBLISHED"] = published
        self.response = response


class TestLookup(unittest.TestCase):
    layer = IMPLICIT_RULESET_REGISTRY_UNIT_TESTING

    def test_no_cache_rule(self):
        view = DummyView()
        request = DummyRequest(view, DummyResponse())
        self.assertEqual(None, DefaultRulesetLookup(view, request)())

    def test_match(self):
        z3c.caching.registry.register(DummyView, "testrule")
        view = DummyView()
        request = DummyRequest(view, DummyResponse())
        self.assertEqual("testrule", DefaultRulesetLookup(view, request)())


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)



from plone.caching.interfaces import ICachingOperation
from plone.caching.operations import Chain
from plone.caching.testing import IMPLICIT_RULESET_REGISTRY_UNIT_TESTING
from plone.registry import field
from plone.registry import Record
from plone.registry import Registry
from plone.registry.interfaces import IRegistry
from zope.component import adapter
from zope.component import provideAdapter
from zope.component import provideUtility
from zope.interface import implementer
from zope.interface import Interface

import unittest


_marker = object()


class DummyView:
    pass


class DummyResponse(dict):
    def addHeader(self, name, value):
        self.setdefault(name, []).append(value)

    def setHeader(self, name, value):
        self[name] = value


class DummyRequest(dict):
    def __init__(self, published, response):
        self["PUBLISHED"] = published
        self.response = response


class TestChain(unittest.TestCase):
    layer = IMPLICIT_RULESET_REGISTRY_UNIT_TESTING

    def setUp(self):
        self.registry = Registry()
        provideUtility(self.registry, IRegistry)

    def test_no_option(self):
        view = DummyView()
        request = DummyRequest(view, DummyResponse())

        chain = Chain(view, request)
        ret = chain.interceptResponse("testrule", request.response)
        chain.modifyResponse("testrule", request.response)

        self.assertEqual(None, ret)
        self.assertEqual({"PUBLISHED": view}, dict(request))
        self.assertEqual({}, dict(request.response))

    def test_operations_list_not_set(self):
        self.registry.records[f"{Chain.prefix}.operations"] = Record(
            field.List(value_type=field.Text())
        )

        view = DummyView()
        request = DummyRequest(view, DummyResponse())

        chain = Chain(view, request)
        ret = chain.interceptResponse("testrule", request.response)
        chain.modifyResponse("testrule", request.response)

        self.assertEqual(None, ret)
        self.assertEqual({"PUBLISHED": view}, dict(request))
        self.assertEqual({}, dict(request.response))

    def test_operations_empty(self):
        self.registry.records[f"{Chain.prefix}.operations"] = Record(
            field.List(value_type=field.Text()), []
        )

        view = DummyView()
        request = DummyRequest(view, DummyResponse())

        chain = Chain(view, request)
        ret = chain.interceptResponse("testrule", request.response)
        chain.modifyResponse("testrule", request.response)

        self.assertEqual(None, ret)
        self.assertEqual({"PUBLISHED": view}, dict(request))
        self.assertEqual({}, dict(request.response))

    def test_chained_operations_not_found(self):
        self.registry.records[f"{Chain.prefix}.operations"] = Record(
            field.List(value_type=field.Text()), ["op1"]
        )

        view = DummyView()
        request = DummyRequest(view, DummyResponse())

        chain = Chain(view, request)
        chain.modifyResponse("testrule", request.response)

        self.assertEqual({"PUBLISHED": view}, dict(request))
        self.assertEqual({}, dict(request.response))

    def test_multiple_operations_one_found(self):
        self.registry.records[f"{Chain.prefix}.operations"] = Record(
            field.List(value_type=field.Text()), ["op1", "op2"]
        )

        view = DummyView()
        request = DummyRequest(view, DummyResponse())

        @implementer(ICachingOperation)
        @adapter(Interface, Interface)
        class DummyOperation:
            def __init__(self, published, request):
                self.published = published
                self.request = request

            def interceptResponse(self, rulename, response):
                return "foo"

            def modifyResponse(self, rulename, response):
                response["X-Mutated"] = rulename

        provideAdapter(DummyOperation, name="op2")

        chain = Chain(view, request)
        ret = chain.interceptResponse("testrule", request.response)

        self.assertEqual("foo", ret)
        self.assertEqual({"PUBLISHED": view}, dict(request))
        self.assertEqual({"X-Cache-Chain-Operations": "op2"}, dict(request.response))

        request = DummyRequest(view, DummyResponse())
        chain = Chain(view, request)
        chain.modifyResponse("testrule", request.response)

        self.assertEqual({"PUBLISHED": view}, dict(request))
        self.assertEqual(
            {"X-Mutated": "testrule", "X-Cache-Chain-Operations": "op2"},
            dict(request.response),
        )

    def test_multiple_operations_multiple_found(self):
        self.registry.records[f"{Chain.prefix}.operations"] = Record(
            field.List(value_type=field.Text()), ["op1", "op2"]
        )

        view = DummyView()
        request = DummyRequest(view, DummyResponse())

        @implementer(ICachingOperation)
        @adapter(Interface, Interface)
        class DummyOperation1:
            def __init__(self, published, request):
                self.published = published
                self.request = request

            def interceptResponse(self, rulename, response):
                return "foo"

            def modifyResponse(self, rulename, response):
                response["X-Mutated-1"] = rulename

        provideAdapter(DummyOperation1, name="op1")

        @implementer(ICachingOperation)
        @adapter(Interface, Interface)
        class DummyOperation2:
            def __init__(self, published, request):
                self.published = published
                self.request = request

            def interceptResponse(self, rulename, response):
                return "bar"

            def modifyResponse(self, rulename, response):
                response["X-Mutated-2"] = rulename

        provideAdapter(DummyOperation2, name="op2")

        chain = Chain(view, request)
        ret = chain.interceptResponse("testrule", request.response)

        self.assertEqual("foo", ret)
        self.assertEqual({"PUBLISHED": view}, dict(request))
        self.assertEqual({"X-Cache-Chain-Operations": "op1"}, dict(request.response))

        request = DummyRequest(view, DummyResponse())
        chain = Chain(view, request)
        chain.modifyResponse("testrule", request.response)

        self.assertEqual({"PUBLISHED": view}, dict(request))
        self.assertEqual(
            {
                "X-Mutated-1": "testrule",
                "X-Mutated-2": "testrule",
                "X-Cache-Chain-Operations": "op1; op2",
            },
            dict(request.response),
        )


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)



from pathlib import Path
from setuptools import setup


version = "3.0.0a1"

long_description = (
    f"{Path('README.rst').read_text()}\n{Path('CHANGES.rst').read_text()}"
)

setup(
    name="plone.caching",
    version=version,
    description="Zope 2 integration for z3c.caching",
    long_description=long_description,
    long_description_content_type="text/x-rst",
    # Get more strings from
    # https://pypi.org/classifiers/
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Framework :: Plone",
        "Framework :: Plone :: 6.2",
        "Framework :: Plone :: Core",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords="plone http caching",
    author="Plone Foundation",
    author_email="plone-developers@lists.sourceforge.net",
    url="https://pypi.org/project/plone.caching",
    license="GPL",
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.10",
    install_requires=[
        "plone.registry",
        "plone.transformchain",
        "Zope",
        "z3c.caching [zcml]",
    ],
    extras_require={
        "test": [
            "plone.testing",
        ],
    },
    entry_points="""
      # -*- Entry points: -*-
      """,
)



from plone.caching.interfaces import _
from plone.caching.interfaces import ICachingOperation
from plone.caching.interfaces import ICachingOperationType
from plone.caching.utils import lookupOptions
from zope.component import adapter
from zope.component import queryMultiAdapter
from zope.interface import implementer
from zope.interface import Interface
from zope.interface import provider


@implementer(ICachingOperation)
@provider(ICachingOperationType)
@adapter(Interface, Interface)
class Chain:
    """Caching operation which chains together several other operations.

    When intercepting the response, the first chained operation to return a
    value will be used. Subsequent operations are ignored. When modifying the
    response, all operations will be called, in order.

    The names of the operations to execute are found in the ``plone.registry``
    option ``plone.caching.operations.chain.operations`` by default, and can
    be customised on a per-rule basis with
    ``plone.caching.operations.chain.${rulename}.chain``.

    The option must be a sequence type (e.g. a ``Tuple``).
    """

    title = _("Chain")
    description = _("Allows multiple operations to be chained together")
    prefix = "plone.caching.operations.chain"
    options = ("operations",)

    def __init__(self, published, request):
        self.published = published
        self.request = request

    def interceptResponse(self, rulename, response):
        options = lookupOptions(self.__class__, rulename)

        chained = []

        if options["operations"]:
            for name in options["operations"]:
                operation = queryMultiAdapter(
                    (self.published, self.request), ICachingOperation, name=name
                )

                if operation is not None:
                    chained.append(name)

                    value = operation.interceptResponse(rulename, response)
                    if value is not None:
                        response.setHeader(
                            "X-Cache-Chain-Operations", "; ".join(chained)
                        )
                        return value

    def modifyResponse(self, rulename, response):
        options = lookupOptions(self.__class__, rulename)

        chained = []

        if options["operations"]:
            for name in options["operations"]:
                operation = queryMultiAdapter(
                    (self.published, self.request), ICachingOperation, name=name
                )

                if operation is not None:
                    chained.append(name)
                    operation.modifyResponse(rulename, response)

        if chained:
            response.setHeader("X-Cache-Chain-Operations", "; ".join(chained))
